// Project Sword & Gun Copyright © 2021 Kevin Poretti

#pragma once

#include "CoreMinimal.h"
#include "GameFramework/Character.h"
#include "SNGCharacterBase.generated.h"

class USpringArmComponent;
class UCameraComponent;

UCLASS()
class SWORDNGUN_API ASNGCharacterBase : public ACharacter
{
	GENERATED_BODY()

public:
	// Sets default values for this character's properties
	ASNGCharacterBase(const FObjectInitializer& ObjectInitializer);

	/** Character input control */
	void MoveForward(float Value);

	void MoveRight(float Value);

	void TurnRate(float Value);

	void LookUpRate(float Value);
	
	FVector GetInputAsWorldDirection();

	FORCEINLINE float GetInputThreshold() { return InputThreshold; } 
	/** End character input control */

	/** Hit-stop */
	void ApplyHitStop(float StopTime, float SlowTime);
	
protected:
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category="Components")
	UCameraComponent* CameraComponent;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category="Components")
	USpringArmComponent* SpringArmComponent;

	UPROPERTY(BlueprintReadOnly)
	FVector2D InputDirection;

	/** Magnitude InputDirection needs to meet for the input to be considered "non-zero" */
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category="Input")
	float InputThreshold;
	
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Animations")
	UAnimMontage* JumpNeutralMontage;
	
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Animations")
	UAnimMontage* JumpFwdMontage;

	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Animations")
	UAnimMontage* JumpBackMontage;
	
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Animations")
	UAnimMontage* DashMontage;
	
	UPROPERTY(BlueprintReadWrite)
	bool CanMove;

	UPROPERTY(BlueprintReadWrite)
	bool CanMovementCancelMontage;

	/** Field of the view the camera starts with */
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category="Field Of View", meta=(ClampMin="30", UIMin="30"))
	float DefaultFOV;

	/** Field of the view when the player starts sprinting */
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category="Field Of View", meta=(ClampMin="30", UIMin="30"))
	float SprintFOV;

	/** */
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category="Field Of View", meta=(ClampMin="0", UIMin="0"))
	float SprintFOVInterpSpeed;
	
	void StartJump();
	
	virtual void Landed(const FHitResult& Hit) override;
	
	// Called when the game starts or when spawned
	virtual void BeginPlay() override;
	
public:	
	// Called every frame
	virtual void Tick(float DeltaTime) override;

	// Called to bind functionality to input
	virtual void SetupPlayerInputComponent(class UInputComponent* PlayerInputComponent) override;
	
};
